// TEST.CPP: Test-application for OWLEXT classes & code
//
#include <owlext/all.h>
#pragma hdrstop

#include <owlext/owlxlink.h>

#include <owl/editfile.h>
#include <owl/statusba.h>
#include <owl/scroller.h>
#include <stdio.h>
#include "test.rh"

OWLEXT_USING_NAMESPACE
	// So we can refer to all OwlExt classes without the prefacing "OwlExt::"


// These classes will test TProgressDlg (code by Rich Goldstein)
//
#define CM_BEGINTHREAD 100
#define CM_ENDTHREAD 101
#define CM_DIALOGTHREAD 102

class SillyThread : public TProgressThread
{
public:
  SillyThread(HWND hWnd, int count)
    : TProgressThread(0, hWnd), _count(count)
  { }

  int Run()
  {
    NotifyStart(_count);
    for (int i = 0; i < _count && !ShouldTerminate(); ++i)
    {
      Sleep(100);
      NotifyProgress(i);
    }
    NotifyEnd();
    return 0;
  }

protected:
  int _count;
};

class SillyProgressDlg : public TThreadProgressDlg
{
public:
  SillyProgressDlg(TWindow* parent)
    : TThreadProgressDlg(parent, "Running Silly Loop")
  { }

  TProgressThread* BuildThread()
  { return new SillyThread(GetHandle(), 50); }
};

class TProgressFrame : public TDecoratedFrame
{
public:
  TProgressFrame(TWindow* parent,
                const char far* title,
                TWindow* clientWnd,
                bool trackMenuSelection = false,
                TModule* module = 0);

protected:
  void CmBeginThread();
  void CmEndThread();
  void CmDialogThread()
    { SillyProgressDlg(this).Execute(); }

  void CleanupWindow();

  LRESULT EvProgressStart(WPARAM, LPARAM range)
  { _gauge->SetRange(0, (int)range); return 0; }
  LRESULT EvProgress(WPARAM, LPARAM value)
  { _gauge->SetValue((int)value); return 0; }
  LRESULT EvProgressEnd(WPARAM, LPARAM)
  {
    int l, h;
    _gauge->GetRange(l, h);
    _gauge->SetValue(l);

    // Wait for thread to end, then delete the thread object
    if (_othread)
    {
      if (_othread->GetStatus() != TProgressThread::Finished)
         _othread->TerminateAndWait(TProgressThread::NoLimit);
      delete _othread;
      // Prevent re-entrancy in 16 bit versions, and multiple
      // instances of the thread in 32 bit versions
      _othread = 0;
    }

    return 0;
  }

  TStatusBar* _status;
  TGaugeGadget*   _gauge;

  TProgressThread* _othread;

  DECLARE_RESPONSE_TABLE(TProgressFrame);
};

DEFINE_RESPONSE_TABLE1(TProgressFrame, TDecoratedFrame)
   EV_MESSAGE(WM_GSC_PROGRESS_START, EvProgressStart),
   EV_MESSAGE(WM_GSC_PROGRESS, EvProgress),
   EV_MESSAGE(WM_GSC_PROGRESS_END, EvProgressEnd),
   EV_COMMAND(CM_BEGINTHREAD, CmBeginThread),
   EV_COMMAND(CM_ENDTHREAD, CmEndThread),
   EV_COMMAND(CM_DIALOGTHREAD, CmDialogThread),
END_RESPONSE_TABLE;

TProgressFrame::TProgressFrame(TWindow* parent,
                const char far* title,
                TWindow* clientWnd,
                bool trackMenuSelection,
                TModule* module)
  : TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module), _othread(0)
{
            _status = new TStatusBar(0, TGadget::Recessed,
                                TStatusBar::CapsLock |
                                TStatusBar::NumLock |
                                TStatusBar::Overtype);
            // Create a gauge control
            _gauge = new TGaugeGadget(1, "%d%%", 96);
            _status->Insert(*_gauge);
            Insert(*_status, TDecoratedFrame::Bottom);
}
void TProgressFrame::CleanupWindow()
{
   if (_othread)
   {
      if (_othread->GetStatus() != TProgressThread::Finished)
         _othread->TerminateAndWait(TProgressThread::NoLimit);
      delete _othread;
   }
   TDecoratedFrame::CleanupWindow();
}
void TProgressFrame::CmEndThread()
{
   if (_othread && _othread->GetStatus() != TProgressThread::Finished)
      _othread->Terminate();
}
void TProgressFrame::CmBeginThread()
{
   if (_othread)
   {
      MessageBox("A thread is already running.");
      return;
   }
   _othread = new SillyThread(GetHandle(), 100);

   // In 16 bit version, be careful to avoid re-entrancy!
   if (_othread)
      _othread->Start();
}


// This window will test TScrollerEx (code by Rich Goldstein)
//
class CircleWindow : public TWindow
{
   public:
      CircleWindow(TWindow* parent, TModule* mod = 0)
         : TWindow(parent, 0, mod)
      {
         Attr.Style |= WS_HSCROLL | WS_VSCROLL; // need scrollers
         Scroller = new TScroller(this,1,1,10,10);
      }

      void SetupWindow()
      {
         TWindow::SetupWindow();
         // Recall that SetUnits expects the scroll units in device units
         TClientDC dc(*this);

         // Set up 1/4 inch scroll units
         Scroller->SetUnits(dc.GetDeviceCaps(LOGPIXELSX)/4, dc.GetDeviceCaps(LOGPIXELSY)/4);
         // We want a 4" x 4" scrolling area
         Scroller->SetRange(15, 15);
      }

      void Paint(TDC& dc, bool, TRect& r)
      {
         // map the dc so that the window origin is 2 inches
         // into the first quadrant, using logical twips
         // Note that the ViewPortOrg is already established by
         // TScroller
         dc.SetMapMode(MM_ANISOTROPIC);
         // Describe Logical Twips
         dc.SetWindowExt(TSize(1440,1440));
         dc.SetViewportExt(TSize(dc.GetDeviceCaps(LOGPIXELSX), dc.GetDeviceCaps(LOGPIXELSY)));
         // We want the origin 2 inches to the right and down
         // DO NOT USE SetViewportOrg!!!!!
         dc.SetWindowOrg(TPoint(-1440 * 2, -1440 * 2));

         TPen pen(TColor::Black, 2);
         TBrush brush(TColor(0,0,196));
         dc.SelectObject(pen);
         dc.SelectStockObject(NULL_BRUSH);
         // Paint an empty rectangle 1/4 inch in from the edges
         dc.Rectangle(-2520, -2520, 2520, 2520);
         dc.SelectObject(brush);
         // Paint a circle centered a 0,0, and 1 inch in radius
         dc.Ellipse(-1440, -1440, 1440, 1440);
         dc.RestoreBrush();
         dc.RestorePen();
      }
};


// This code tests TEditList (code by Chris Kohloff)
//
#include <owlext/editlist.h>
class TEditListFrame : public TFrameWindow
{
  public:
    TEditListFrame(TWindow* parent = 0);
    virtual void SetupWindow();
  protected:
    TEditList* List;
};

TEditListFrame::TEditListFrame(TWindow* parent)
: TFrameWindow(parent, "EditList Application",
               List = new TEditList(0, 0, 0, 0, 0, 0))
{
  List->Attr.Style |= LBS_NOINTEGRALHEIGHT;
  List->Attr.Style &= ~LBS_SORT;
}

void TEditListFrame::SetupWindow()
{
  TFrameWindow::SetupWindow();

  char text[256];
  for (int i = 0; i < 20; i++)
  {
    sprintf(text, "This is item number %d", i);
    List->AddString(text);
  }
}


// This will be for our "main" framewindow
//
class TNoClientWindow : public TWindow
{
public:
  TNoClientWindow(TWindow* parent = 0)
  	 : TWindow(parent)
  { Attr.W = 400; Attr.H = 0; }
};


// TTestWindow: for our TTabWindow client
//
class TTestWindow : public TWindow
{
public:
    TTestWindow(const char* testText, int x, int y)
        : TWindow(0, testText)
    {
        theStatic = new TStatic(this, -1, testText, x, y, 200, 24);
    }
    ~TTestWindow()
    { }

    TStatic* theStatic;
};


// We're going to create several framewindows, just to show we can, as well
// as to highlight the various TFramewindow types in OWLEXT.
//
class TOwlExtApp : public TApplication
{
public:
	TOwlExtApp();
  ~TOwlExtApp();

protected:
	virtual void InitMainWindow();

protected:
  void CmControlsDlg();
};
TModule* module = 0;
TOwlExtApp::TOwlExtApp()
  : TApplication("Test",module)
{ }
TOwlExtApp::~TOwlExtApp()
{ }
void TOwlExtApp::InitMainWindow()
{
  // I want the main window to be a caption & menubar, nothing else
	TMultiFrame* mainWin = new TMultiFrame(0, "OWLEXT Showcase", new TNoClientWindow, true);
  mainWin->AssignMenu(MAINMENU);
  mainWin->Attr.X = 0;
  mainWin->Attr.Y = 0;

  // Let's add a new top-level window
  TFrameWindow* frame1 = new TFrameWindow(0, "TTabWindow Demo");
    TTabWindow* frame1Client = new TWin95TabWindow(frame1);
      TWindow* array[] =
      {
        new TTestWindow("Test 1", 170, 20),
        new TTestWindow("Test Window 2", 270, 40),
        new TTestWindow("Test 3", 0, 60),
        0
      };
    frame1Client->Attach(array);
  frame1->SetClientWindow(frame1Client);
  frame1->GetClientWindow()->Attr.W = 350;
  frame1->GetClientWindow()->Attr.H = 200;
  frame1->Attr.X = 0;
  frame1->Attr.Y = 25;
  mainWin->AddFrame(frame1);

  // Let's add a new top-level window, this one to showcase TScrollerEx
  TFrameWindow* frame2 = new TFrameWindow(0, "TScrollerEx Demo", new CircleWindow(0), true);
  frame2->GetClientWindow()->Attr.W = 350;
  frame2->GetClientWindow()->Attr.H = 200;
  frame2->Attr.X = 0;
  frame2->Attr.Y = 50;
  mainWin->AddFrame(frame2);

  // Add a new top-level window to test TProgressDlg & company
  TFrameWindow* frame3 = new TProgressFrame(0, "TProgressDlg Demo", new TWindow(0, ""), true);
  frame3->AssignMenu(PROGRESSMENU);
  frame3->GetClientWindow()->Attr.W = 350;
  frame3->GetClientWindow()->Attr.H = 200;
  frame1->Attr.X = 0;
  frame1->Attr.Y = 75;
  mainWin->AddFrame(frame3);

  // Add a new top-level window to test TEditList
  TFrameWindow* frame4 = new TEditListFrame;
  frame4->Attr.X = 0;
  frame4->Attr.Y = 100;
  mainWin->AddFrame(frame4);

  SetMainWindow(mainWin);
}
void TOwlExtApp::CmControlsDlg()
{
}

// Simple OwlMain, since everything's in TOwlExtApp
int OwlMain(int, char**)
{ return TOwlExtApp().Run(); }

